#Licensed Materials - Property of IBM
#5724-M24
#Copyright IBM Corporation 2012. All Rights Reserved.
#US Government Users Restricted Rights - Use, duplication or disclosure
#restricted by GSA ADP Schedule Contract with IBM Corp.

import getopt

def getTargetsForApplication(application):
    targets = []
    deployment = AdminConfig.getid('/Deployment:%s/' % application);
    deploymentTargets = AdminUtilities.convertToList(AdminConfig.showAttribute(deployment, 'deploymentTargets'))
    for deploymentTarget in deploymentTargets:
        if AdminConfig.getObjectType(deploymentTarget) == 'ClusteredTarget':
            clusterName = AdminConfig.showAttribute(deploymentTarget, 'name')
            cluster = AdminConfig.getid('/ServerCluster:%s/' % clusterName)
            if cluster not in targets:
                targets.append(cluster)
        elif AdminConfig.getObjectType(deploymentTarget) == 'ServerTarget':
            serverName = AdminConfig.showAttribute(deploymentTarget, 'name')
            nodeName = AdminConfig.showAttribute(deploymentTarget, 'nodeName')
            print "Application " + application + " is deployed to server " + serverName + " on node " + nodeName + "."
            server = AdminConfig.getid('/Node:%s/Server:%s/' % (nodeName, serverName))
            if server not in targets:
                targets.append(server)
    return targets

#------------------------------------------------------------------------------
# Get clusters by topologyName
#------------------------------------------------------------------------------
def getClusters(topologyName):	
	topologyClusters = []	
	
	# identify the topology's clusters (in a certain order of preference)
	for role in ["WebApp", "Support", "AppTarget", "Messaging"]:
		wbiCluster = AdminConfig.getid('/WBITopology:%s/WBITopologyRole:%s/WBICluster:/' % (topologyName, role))
		if wbiCluster:
			clusterName = AdminConfig.showAttribute(wbiCluster, 'name')
			cluster = AdminConfig.getid('/ServerCluster:%s' % clusterName);
			if cluster not in topologyClusters:
				topologyClusters.append(cluster)
	return 	topologyClusters
	
#------------------------------------------------------------------------------
# Map web modules to the virtual host that corresponds to the virtual host
#------------------------------------------------------------------------------
def updageVH(virtualHostName, topologyName):	
	topologyClusters = getClusters(topologyName)
	applications = AdminApp.list().splitlines()

	# map web modules to the virtual host that corresponds to the virtual hosts    
	for application in applications:
	    for target in getTargetsForApplication(application):
	        if target in topologyClusters:
	            print "Mapping web modules in application " + application + " to virturlHost " + virtualHostName
	            AdminApp.edit(application, ['-MapWebModToVH', [['.*', '.*', virtualHostName]]])
	            break;

#------------------------------------------------------------------------------
# Print script usage
#------------------------------------------------------------------------------
def usage():
	print
	print "Usage:"
	print
	print "   updateVirtualHost(.bat|.sh) [OPTIONS]"
	print
	print "   -d, --deployment-environment=DE_NAME1,DE_NAME2 Specify the deployment"
	print "                                        environment names."
	print "   -v, --virtual-host-name=virtual_host_name virtual host on which to map web modules"
	print "                                        in applications"
	print "       --no-sync                        Do not force the immediate propagation"
	print "                                        of changes across nodes."
	print "       --no-save                        Do not save configuration changes."
	print "       --help                           Display this help"
	print
	
#------------------------------------------------------------------------------
# Main entry point
#------------------------------------------------------------------------------	
def main():
	try:
		options, args = getopt.getopt(sys.argv, "d:v:", ["no-save","no-sync", "deployment-environment=", "virtual-host-name=", "help"])
	except getopt.GetoptError, err:
		print str(err)
		usage()
		return 2

	if args:
		usage()
	
	save = 1
	sync = 1
	topologyNames = None
	virtualHostName = None
	for option, arg in options:
		if option in ("-d", "--deployment-environment"):
			topologyNames = arg.split(',')
		elif option in ("-v", "--virtual-host-name"):
			virtualHostName = arg
		elif option == "--no-save":
			save = 0
			sync = 0
		elif option == "--no-sync":
			sync = 0
		elif option == "--help":
			usage()
			return
		else:
			assert 0, "un-handled option"
	
	if  topologyNames == None:
		usage()
	
	if  virtualHostName == None:
		usage()
	
	# map web modules to the virtual host that corresponds to the virtual host
	for topologyName in topologyNames:
		topology = AdminConfig.getid('/WBITopology:%s/' % topologyName)
		if topology:
			if AdminConfig.showAttribute(topology, 'status') == "Configured":
				updageVH(virtualHostName, topologyName)
			else:
				raise "The deployment environment " + topologyName + " has not been configured."
		else:
			raise "Deployment environment " + topologyName + " does not exist."
	
	# save and sync
	if save:
		print "Configuration is being saved."
		AdminConfig.save()
		if sync:
			print "Synchronizing the configuration across all nodes."
			AdminNodeManagement.syncActiveNodes()
		print "Done."
		

if __name__ == "__main__":
	sys.exit(main())
